package compbio.runner.structure;


import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;
import static org.testng.Assert.fail;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;

import javax.xml.bind.ValidationException;

import org.apache.log4j.*;
import org.ggf.drmaa.DrmaaException;
import org.ggf.drmaa.JobInfo;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import compbio.data.sequence.RNAStructScoreManager;
import compbio.data.sequence.Score;
import compbio.data.sequence.ScoreManager;
import compbio.data.sequence.ScoreManager.ScoreHolder;
import compbio.data.sequence.SequenceUtil;
import compbio.engine.AsyncExecutor;
import compbio.engine.Configurator;
import compbio.engine.SyncExecutor;
import compbio.engine.client.ConfExecutable;
import compbio.engine.client.ConfiguredExecutable;
import compbio.engine.client.Executable;
import compbio.engine.client.RunConfiguration;
import compbio.engine.cluster.drmaa.ClusterUtil;
import compbio.engine.cluster.drmaa.JobRunner;
import compbio.engine.cluster.drmaa.StatisticManager;
import compbio.engine.local.AsyncLocalRunner;
import compbio.engine.local.LocalExecutorService;
import compbio.engine.local.LocalRunner;
import compbio.metadata.AllTestSuit;
import compbio.metadata.JobExecutionException;
import compbio.metadata.JobStatus;
import compbio.metadata.JobSubmissionException;
import compbio.metadata.LimitsManager;
import compbio.metadata.PresetManager;
import compbio.metadata.ResultNotAvailableException;
import compbio.metadata.RunnerConfig;
import compbio.runner.msa.ClustalW;
import compbio.runner.structure.RNAalifold;

public class RNAalifoldTester {

	private static Logger log = Logger
			.getLogger(AllTestSuit.RUNNER_TEST_LOGGER);
	
	
	private RNAalifold rnaalifold;
	
	static final String rnaalifoldConfigFile = AllTestSuit.TEST_DATA_PATH
			+ "RNAalifoldParameters.xml";
	public static String test_outfile = "rnaalifold.out";
	
	
	@Test(groups = { AllTestSuit.test_group_runner })
	public void testRunLocally() {
		RNAalifold rnaalifold = new RNAalifold();
		rnaalifold.setInput(AllTestSuit.test_input_aln).setOutput(test_outfile);
		try{
			
			ConfiguredExecutable<RNAalifold> confRNAalifold = Configurator
					.configureExecutable(rnaalifold, Executable.ExecProvider.Local);
			LocalRunner lr = new LocalRunner(confRNAalifold);
			lr.executeJob();
			confRNAalifold = (ConfiguredExecutable<RNAalifold>) lr.waitForResult();
			
			System.out.println("TEST");
			System.out.println(((RNAStructScoreManager) confRNAalifold.getResults()).toString());
			
			assertNotNull(confRNAalifold.getResults()); 
		} catch (JobSubmissionException e) {
			e.printStackTrace();
			fail(e.getLocalizedMessage());
		} catch (JobExecutionException e) {
			e.printStackTrace();
			fail(e.getLocalizedMessage());
		} catch (ResultNotAvailableException e) {
			e.printStackTrace();
			fail(e.getLocalizedMessage());
		}
	}
	
	
	public static void main(String[] args) throws JobSubmissionException,
			JobExecutionException, InterruptedException, ResultNotAvailableException {

		
		log.warn("Logger test :- Run RNAalifold.main()");

		RNAalifold rnaalifold = new RNAalifold();
		rnaalifold.setInput(AllTestSuit.test_input_aln).setOutput("test_outfile.txt");
		
		ConfiguredExecutable<RNAalifold> confRNAalifold = Configurator
					.configureExecutable(rnaalifold);
		AsyncExecutor lr = new AsyncLocalRunner();
		lr.submitJob(confRNAalifold);
		
		
		System.out.println(((RNAStructScoreManager) confRNAalifold.getResults()).toString());

		
		Thread.sleep(3000);
		LocalExecutorService.shutDown();
		
	}
	
}
